@@ -119,14 +119,15 @@ use constant IS_CGI      => IS_MOD_PERL || exists $ENV{'GATEWAY_INTERFACE'};
 # we use a Hash to store the configuration. This is simpler to pass to functions.
 our %config;
 
-$config{version}                  = "24.07.12";  # year.month.day
+$config{version}                  = "25.01.25";  # year.month.day
 
 my $dirname = dirname(__FILE__);
 
-if (-e '/etc/qemu-web-desktop/config.pl') {
-  require '/etc/qemu-web-desktop/config.pl';
-} elsif (-e "$dirname/../config.pl") {
+# use local config when found
+if (-e "$dirname/../config.pl") {
   require "$dirname/../config.pl";
+} elsif (-e '/etc/qemu-web-desktop/config.pl') {
+  require '/etc/qemu-web-desktop/config.pl';
 } else {
   die "Can not find config.pl (e.g. /etc/qemu-web-desktop/config.pl or $dirname/../config.pl";
 }
@@ -158,15 +159,22 @@ $config{oneshot}                  = 0;
 
 for(my $i = 0; $i < @ARGV; $i++) {
   $_ = $ARGV[$i];
-  if(/--help|-h|--version|-v$/) {
+  if(/--help|-h$/) {
     print STDERR "$0: launch a QEMU/KVM machine in a browser window. Version $config{version}\n\n";
     print STDERR "Usage: $0 --option1=value1 ...\n\n";
     print STDERR "Valid options are:\n";
-    foreach my $key (keys %config) {
+    foreach my $key (sort keys %config) {
       print STDERR "  --$key=VALUE [$config{$key}]\n";
     }
     print "\n(c) 2020- Emmanuel Farhi - GRADES - Synchrotron Soleil. AGPL3.\n";
     exit;
+  } elsif (/--version|-v$/) {
+    print STDERR "$0 $config{version}\n\n";
+    print STDERR "Copyright (C) 2020-     Synchrotron Soleil,     GPL3.\n";
+    print STDERR "This is free software; see the source for copying conditions.  There is NO\n";
+    print STDERR "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n";
+    print STDERR "Written by Emmanuel Farhi <emmanuel.farhi\@synchrotron-soleil.fr>\n";
+    exit;
   } elsif (/^--(\w+)=(\w+)$/) {      # e.g. '--opt=value'
     if (exists($config{$1})) {
       $config{$1} = $2;
@@ -625,13 +633,19 @@ if (not $error) {
     $output   .= "<li>$ok Will use ISO from ";
   } else {
     # qemu-img now requires to specify master "backing" file format with '-F FMT'
-    my $img_info = `qemu-img info $config{dir_machines}/$session{machine}`;
-    my @backing_format = $img_info =~ /file format:\s*([a-z0-9]+)/i;
-    my $F     = $backing_format[0];
-    $cmd      = "qemu-img create -F $F -b $config{dir_machines}/$session{machine}"
-              . " -f qcow2 $session{snapshot}";
-    $res      = `$cmd`; # execute command
-    $output  .= "<li>$ok Creating snapshot from ";
+    if ($config{snapshot_use_master} and -w "$config{dir_machines}/$session{machine}") {
+      # option to NOT use snapshot. Work on master when writable.
+      $session{snapshot} = "$config{dir_machines}/$session{machine}";
+      $output  .= "<li>$ok Using master ";
+    } else {
+      my $img_info = `qemu-img info $config{dir_machines}/$session{machine}`;
+      my @backing_format = $img_info =~ /file format:\s*([a-z0-9]+)/i;
+      my $F     = $backing_format[0];
+      $cmd      = "qemu-img create -F $F -b $config{dir_machines}/$session{machine}"
+                . " -f qcow2 $session{snapshot}";
+      $res      = `$cmd`; # execute command
+      $output  .= "<li>$ok Creating snapshot from ";
+    }
   }
   $output .= "<a href='$config{http}://$session{server_name}/$config{service}/machines/$session{machine}' target=_blank>"
   . "$session{machine}</a> as session $session{name}</li>\n";
@@ -967,8 +981,8 @@ if (not $error) {
   
   print STDERR $0.": $session{name}: $session{date}: START $session{machine} by $session{user} $session{user_email}\n";
   print STDERR $0.": $session{name}: json:  $session{json}\n";
-  print STDERR $0.": $session{name}: URL:   $session{url}\n";
   print STDERR $0.": $session{name}: PIDs:  @{$session{pid}}\n" if ($session{pid});
+  print STDERR $0.": $session{name}: URL:   $session{url}\n";
 } else {
   print STDERR $0.": $session{name}: $session{date}: ERROR $error\n";
 }
@@ -1259,7 +1273,7 @@ sub service_housekeeping {
   my $now         = localtime();
   foreach my $snapshot (glob("$dir/$service"."_*")) {
     
-    if (-d $snapshot && time - (stat($snapshot))[9] > $config{boot_delay}) { # is a snapshot directory
+    if (-d $snapshot && time - (stat($snapshot))[9] > 10*$config{boot_delay}) { # is a snapshot directory
       my $snaphot_name = fileparse($snapshot); # just the session name
       
       if (not -e "$cfg/$snaphot_name.json") {
@@ -1287,7 +1301,7 @@ sub service_housekeeping {
     
     # session exists, load JSON
     my $session_ref = session_load(\%config, $json);
-    if ($session_ref && time - (stat($json))[9] > $config{boot_delay}) {
+    if ($session_ref && time - (stat($json))[9] > 10*$config{boot_delay}) {
       my %session = %{ $session_ref };
       if (not -d $session{dir_snapshot}) { 
         print STDERR $0." [$now] housekeeping: $json (left-over, no snapshot)\n";
@@ -1376,7 +1390,15 @@ sub service_monitor {
               my @pids = @{ $session{pid} };
               $out .= "<tr>";
               $out .= "<td>$session{date}</td>";
-              $out .= "<td>$session{name}</td>";
+              if ($iter == 2) { # admin only. Appear as a collapsible text
+                $out .= "<td><details><summary>$session{name}</summary><p><small>";
+                my %hash = %session;
+                $hash{password}="XXXX"; # hide user pw
+                $out .= "<b>$_</b>=$hash{$_}; " for (keys %hash);
+                $out .= "</small></p></details>";
+              } else {
+                $out .= "<td>$session{name}</td>";
+              }
               $out .= "<td><a href='$config{http}://$session{server_name}/$config{service}/machines/$session{machine}'>$session{machine}</a></td>";
               if ($session{user_email}) {
                 $out .= "<td><a href='mailto:$session{user_email}'>$session{user}</a></td>";
